#!/usr/bin/perl
#
# This is an enhanced version of Bob Backstrom's "makems.pl"
#
# The path where the binaries are:
$GGNFS_BIN_PATH="../src";

# The path where gzip and gunzip are to be found (for DOS only):
$SYS_BIN_PATH="c:/cygwin/bin";
# Other possible choices include:
#$GGNFS_BIN_PATH="c:/mingw/msys/1.0/bin";
#$GGNFS_BIN_PATH=""; Use the null string if a DOS version of gzip has been installed.

if($^O ne "MSWin32") {
  $dir = "ls -t";
  $GUNZIP="gunzip";
}
else {
  $dir = "dir /b /o-d";
  $GUNZIP=$SYS_BIN_PATH."/gzip -d";
}

$FILE2 = "msieve.dat";
system "$dir *.fb > $FILE2";
open(RD, $FILE2);
$rr = <RD>;
if($rr eq "msieve.fb\n") {
$rr = <RD>; }
$NAME = substr($rr,0,length($rr)-4);

$FILE1 = "$NAME.poly";
$FILE3 = "msieve.fb"; # Factor base file
$MSIEVE = "msieve -nc -v"; # Msieve NFS, verbose
$FILE4 = "worktodo.ini"; # Work to do file

$r[0] = "0";
$r[1] = "0";

$deg = 0;

open(IN, $FILE1); # Find poly degree

while($line = <IN>)
{
$line = lc($line);

if($line =~ /^\s*c(\d+)/)
{
if($deg < $1) { $deg = $1 }
}
}

if(!$deg)
{
close(IN);
print "Coeffs \"cn: [-]ddd\" not found in \"$FILE1\"\n";
exit 1;
}

for($i = 0; $i <= $deg; $i++){ $c[$i] = "0"; }

print "\ndeg: $deg\n";

seek(IN, 0, 0); # rewind input file
open(OUT1, ">$FILE2"); # dat file
open(OUT2, ">$FILE3"); # fb file
open(OUT3, ">$FILE4"); # worktodo file

while($line = <IN>)
{
$line = lc($line); # lower case

if($line =~ /^\s*n\s*:\s*(\d+)/) # n: ddd
{
$n = $1;
}
elsif($line =~ /^\s*m\s*:\s*(\d+)/) # m: ddd
{
$r[0] = "-" . $1; # (no arith, just strings)
$r[1] = 1;
}
elsif($line =~ /^\s*y(\d+)\s*:\s*(-*)(\d+)/)
{
$r[$1] = $2 . $3; # yi: [-]ddd
}

elsif($line =~ /^\s*c(\d+)\s*:\s*(-*)(\d+)/)
{
$c[$1] = $2 . $3; # ci: [-]ddd
}

elsif($line =~ /^\s*rlim\s*:\s*(\d+)/) # rlim: ddd
{
$frmax = $1;
}
elsif($line =~ /^\s*alim\s*:\s*(\d+)/) # alim: ddd
{
$famax = $1;
}

elsif($line =~ /^\s*lpbr\s*:\s*(\d+)/) # lpbr: ddd
{
$srtfbits = $1;
}
elsif($line =~ /^\s*lpba\s*:\s*(\d+)/) # lpba: ddd
{
$satfbits = $1;
}
}

print OUT1 "N $n\n"; # N ddd to dat
print OUT3 "$n\n"; # ddd to worktodo

print OUT2 "N $n\n"; # N ddd to fb
print "N $n\n";

$s = "R0 $r[0]\n" .
"R1 $r[1]\n";

print OUT2 $s; # R0, R1 to fb
print $s;

for($i = 0, $s = ""; $i <= $deg; $i++)
{
$s = $s . "A$i $c[$i]\n";
}

print OUT2 $s; # A0 - A{deg} to fb
print $s;

$s = "FRMAX $frmax\n" .
"FAMAX $famax\n" .
"SRTFBITS $srtfbits\n" .
"SATFBITS $satfbits\n";

print OUT2 $s; # FRMAX etc to fb
print $s;

close(IN);

close(OUT2);
close(OUT3);

if(-e "spairs.save.gz") {
$cmd = "$GUNZIP -c spairs.save.gz >> $FILE2";
print "==> $cmd\n";
system($cmd); } # Append rels to dat
else {
$PROCRELS = $GGNFS_BIN_PATH."/procrels";
chop $rr;
$cmd = "$PROCRELS -prel rels.bin -fb $rr -dump";
print "==> $cmd\n";
system($cmd);

$i=0;
$j=1;
undef $/;
open IN, "spairs.dump.0";
print "Appending to msieve.dat\n";
while($j) {
print OUT1 <IN>;
close IN;
$i++;
open(IN, "spairs.dump.$i") || ($j=0); }
}

$cmd = "$MSIEVE";

print "==> $cmd\n";
system($cmd); # Run Msieve
